/*******************************************************************************
* File Name: PGA_Inv_2.c  
* Version 1.10
*
*  Description:
*    This file provides the source code to the API for the inverting PGA 
*    User Module.
*
*   Note:
*
*******************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/




#include "cytypes.h"
#include "PGA_Inv_2.h"

#if (PGA_Inv_2_ACLK_ENABLE)
    #include "PGA_Inv_2_ScBoostClk.h"
#endif

/* Constant array for gain settings */
const uint8 PGA_Inv_2_GainArray[10] = { 
   (PGA_Inv_2_PGA_INV_GAIN_01 | PGA_Inv_2_BIAS_LOW), /* G=-1  */
   (PGA_Inv_2_PGA_INV_GAIN_03 | PGA_Inv_2_BIAS_LOW), /* G=-3  */
   (PGA_Inv_2_PGA_INV_GAIN_07 | PGA_Inv_2_BIAS_LOW), /* G=-7  */
   (PGA_Inv_2_PGA_INV_GAIN_15 | PGA_Inv_2_BIAS_LOW), /* G=-15 */
   (PGA_Inv_2_PGA_INV_GAIN_22 | PGA_Inv_2_BIAS_LOW), /* G=-22 */
   (PGA_Inv_2_PGA_INV_GAIN_23 | PGA_Inv_2_BIAS_LOW), /* G=-23 */
   (PGA_Inv_2_PGA_INV_GAIN_24 | PGA_Inv_2_BIAS_LOW), /* G=-24 */
   (PGA_Inv_2_PGA_INV_GAIN_31 | PGA_Inv_2_BIAS_LOW), /* G=-31 */
   (PGA_Inv_2_PGA_INV_GAIN_47 | PGA_Inv_2_BIAS_LOW), /* G=-47 */
   (PGA_Inv_2_PGA_INV_GAIN_49 | PGA_Inv_2_BIAS_LOW)  /* G=-49 */
};

/* Constant array for gain compenstion settings */
const uint8 PGA_Inv_2_GainComp[10] = { 
   ( PGA_Inv_2_COMP_4P35PF  | ( PGA_Inv_2_REDC_00 >> 2 )), /* G=-1  */
   ( PGA_Inv_2_COMP_4P35PF  | ( PGA_Inv_2_REDC_01 >> 2 )), /* G=-3  */
   ( PGA_Inv_2_COMP_3P0PF   | ( PGA_Inv_2_REDC_01 >> 2 )), /* G=-7  */
   ( PGA_Inv_2_COMP_3P0PF   | ( PGA_Inv_2_REDC_01 >> 2 )), /* G=-15 */
   ( PGA_Inv_2_COMP_3P6PF   | ( PGA_Inv_2_REDC_01 >> 2 )), /* G=-22 */
   ( PGA_Inv_2_COMP_3P0PF   | ( PGA_Inv_2_REDC_11 >> 2 )), /* G=-23 */
   ( PGA_Inv_2_COMP_3P0PF   | ( PGA_Inv_2_REDC_10 >> 2 )), /* G=-24 */
   ( PGA_Inv_2_COMP_3P6PF   | ( PGA_Inv_2_REDC_11 >> 2 )), /* G=-31 */
   ( PGA_Inv_2_COMP_3P6PF   | ( PGA_Inv_2_REDC_00 >> 2 )), /* G=-47 */
   ( PGA_Inv_2_COMP_3P6PF   | ( PGA_Inv_2_REDC_00 >> 2 ))  /* G=-49 */
};

uint8 PGA_Inv_2_initVar = 0;

/*******************************************************************************
* Function Name: PGA_Inv_2_Start
********************************************************************************
* Summary:
*  The start function initializes the inverting PGA with the default values, and sets
*  the power to the given level.  A power level of 0, is the same as executing
*  the stop function.
*
* Parameters:  
*  power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void PGA_Inv_2_Start(void) 
{

   if(PGA_Inv_2_initVar == 0)
   {
      PGA_Inv_2_initVar = 1;

      /* Set PGA mode */
      PGA_Inv_2_CR0 = PGA_Inv_2_MODE_PGA;  
	
      /* Set inverting PGA mode  and reference mode */
      PGA_Inv_2_CR1 &= ~PGA_Inv_2_PGA_INV;  

      /* Set gain and compensation */
      PGA_Inv_2_SetGain(PGA_Inv_2_DEFAULT_GAIN);

      /* Set power */
      PGA_Inv_2_SetPower(PGA_Inv_2_DEFAULT_POWER);
   }

   /* If a boost clock is required  */
   #if (PGA_Inv_2_ACLK_ENABLE)
      PGA_Inv_2_ACLK_PWRMGR |= PGA_Inv_2_ACLK_ACT_PWR_EN;
      PGA_Inv_2_ScBoostClk_Enable();  // turn on boost pump
	   PGA_Inv_2_BST |= 0x08;
   #endif 

   /* Enable power to the Amp */
   PGA_Inv_2_PWRMGR |= PGA_Inv_2_ACT_PWR_EN;
}

/*******************************************************************************
* Function Name: PGA_Inv_2_Stop
********************************************************************************
* Summary:
*  Summary of function.
*
* Parameters:  
*  Powers down amplifier to lowest power state.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void PGA_Inv_2_Stop(void)
{
   /* Disble power to the Amp */
   PGA_Inv_2_PWRMGR &= ~PGA_Inv_2_ACT_PWR_EN;

   /* If a boost clock is required  */
   /* Disable and power down boost clock */
   #if (PGA_Inv_2_ACLK_ENABLE)
      PGA_Inv_2_ACLK_PWRMGR &= ~PGA_Inv_2_ACLK_ACT_PWR_EN;
      PGA_Inv_2_ScBoostClk_Disable();  
   #endif 
}

/*******************************************************************************
* Function Name: PGA_Inv_2_SetPower
********************************************************************************
* Summary:
*  Set the power of the PGA
*
* Parameters:  
*  power:   Sets power level between (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void PGA_Inv_2_SetPower(uint8 power) 
{
   uint8 tmpCR;

   tmpCR = PGA_Inv_2_CR1 & ~PGA_Inv_2_DRIVE_MASK;
   tmpCR |= (power & PGA_Inv_2_DRIVE_MASK);
   PGA_Inv_2_CR1 = tmpCR;  
}

/*******************************************************************************
* Function Name: PGA_Inv_2_SetGain
********************************************************************************
* Summary:
*  This function sets values of the input and feedback resistors to set a 
*  specific gain of the amplifier.
*
* Parameters:  
*  gain:  Sets gain of amplifier.     
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void PGA_Inv_2_SetGain(uint8 gain)
{
    /* Only set new gain if it is a valid gain */
   if( gain <= PGA_Inv_2_GAIN_MAX)
   {
      /* Clear resistors, redc, and bias */
      PGA_Inv_2_CR2 &= ~(PGA_Inv_2_RVAL_MASK | PGA_Inv_2_R20_40B_MASK | 
                                PGA_Inv_2_REDC_MASK | PGA_Inv_2_BIAS_MASK );

      /* Set gain value resistors, redc comp, and bias */
      PGA_Inv_2_CR2 |= (PGA_Inv_2_GainArray[gain] |
                             ( (PGA_Inv_2_GainComp[gain] << 2 ) & PGA_Inv_2_REDC_MASK) );
            
      /* Clear sc_comp  */
      PGA_Inv_2_CR1 &= ~PGA_Inv_2_COMP_MASK;
      /* Set sc_comp  */
      PGA_Inv_2_CR1 |= ( PGA_Inv_2_GainComp[gain] | PGA_Inv_2_COMP_MASK );
   }
}


/* [] END OF FILE */
