/*******************************************************************************
* File Name: SWITCH1.c  
* Version 1.20
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "SWITCH1.h"


/*******************************************************************************
* Function Name: SWITCH1_Write
********************************************************************************
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  void 
*  
*******************************************************************************/
void SWITCH1_Write(uint8 value)
{
    uint8 staticBits = SWITCH1_DR & ~SWITCH1_MASK;
    SWITCH1_DR = staticBits | ((value << SWITCH1_SHIFT) & SWITCH1_MASK);
}


/*******************************************************************************
* Function Name: SWITCH1_SetDriveMode
********************************************************************************
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to this drive mode.
*
* Return: 
*  void
*
*******************************************************************************/
void SWITCH1_SetDriveMode(uint8 mode)
{
	CyPins_SetPinDriveMode(SWITCH1_0, mode);
}


/*******************************************************************************
* Function Name: SWITCH1_Read
********************************************************************************
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  void 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro SWITCH1_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 SWITCH1_Read(void)
{
    return (SWITCH1_PS & SWITCH1_MASK) >> SWITCH1_SHIFT;
}


/*******************************************************************************
* Function Name: SWITCH1_ReadDataReg
********************************************************************************
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  void 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 SWITCH1_ReadDataReg(void)
{
    return (SWITCH1_DR & SWITCH1_MASK) >> SWITCH1_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(SWITCH1_INTSTAT) 

    /*******************************************************************************
    * Function Name: SWITCH1_ClearInterrupt
    ********************************************************************************
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  void 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 SWITCH1_ClearInterrupt(void)
    {
        return (SWITCH1_INTSTAT & SWITCH1_MASK) >> SWITCH1_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */ 
