/*******************************************************************************
* File Name: tx.c  
* Version 1.20
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "tx.h"


/*******************************************************************************
* Function Name: tx_Write
********************************************************************************
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  void 
*  
*******************************************************************************/
void tx_Write(uint8 value)
{
    uint8 staticBits = tx_DR & ~tx_MASK;
    tx_DR = staticBits | ((value << tx_SHIFT) & tx_MASK);
}


/*******************************************************************************
* Function Name: tx_SetDriveMode
********************************************************************************
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to this drive mode.
*
* Return: 
*  void
*
*******************************************************************************/
void tx_SetDriveMode(uint8 mode)
{
	CyPins_SetPinDriveMode(tx_0, mode);
}


/*******************************************************************************
* Function Name: tx_Read
********************************************************************************
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  void 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro tx_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 tx_Read(void)
{
    return (tx_PS & tx_MASK) >> tx_SHIFT;
}


/*******************************************************************************
* Function Name: tx_ReadDataReg
********************************************************************************
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  void 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 tx_ReadDataReg(void)
{
    return (tx_DR & tx_MASK) >> tx_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(tx_INTSTAT) 

    /*******************************************************************************
    * Function Name: tx_ClearInterrupt
    ********************************************************************************
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  void 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 tx_ClearInterrupt(void)
    {
        return (tx_INTSTAT & tx_MASK) >> tx_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */ 
